{% extends "base.html" %}
{% block content %}

<div class="main stats">
  <h1 id="stats-header">High priority incidents per week</h1>
  <div class="module stats-module col-md-12 col-sm-12 col-xs-12">
    <table id="stats-table" class="dataTable">
    </table>
  </div>
</div>

{% raw %}
<script id="stats-table-template" type="text/x-handlebars-template">
  <tbody>
  {{#each .}}
    {{#each .}}
          <tr>
             <th><h3>{{unixToDate @key}}</h3></th>
             <td>
             {{#each .}}
             <tbody>
              {{#each .}}
                    <tr>
                      <th>{{@key}}</th>
                      <td class="stat-value">{{this}}</td>
                    </tr>
              {{/each}}
             </td>
             </tbody>
             {{/each}}
          </tr>
    {{/each}}
    {{/each}}
  </tbody>
</script>
{% endraw %}

{% endblock %}
