/*
  Copyright (c) LinkedIn Corporation. All rights reserved. Licensed under the BSD-2 Clause license.
  See LICENSE in the project root for license information.
*/


/*
 * Base structure
 */


/* Move down content because we have a fixed navbar that is 50px tall */

body {
    font-family: helvetica-light, helvetica;
    background: #fafafa;
}

header {
    height: 50px;
}

header[data-page="error"] {
    display: none;
}

.container-fluid {
    max-width: 1180px;
    margin: 0 auto;
}

.main {
    margin-top: 20px;
}

.main h3:first-child {
    margin-top: 0;
}

#navbar {
    background-color: #222;
    color: #fff;
    font-size: 14px;
}

.navbar .dropdown .dropdown-toggle {
    padding: 15px;
    cursor: pointer;
}

.navbar-toggle {
    float: left;
    border: 1px solid #fff;
}

.navbar-fixed-top .navbar-collapse {
    height: 310px;
}

.navbar .dropdown-menu>li>a:focus,
.navbar .dropdown-menu>li>a:hover {
    background: #00b0e3;
    color: #fff;
}

.navbar-nav li:after {
    background: #008cc9;
    bottom: 0;
    content: "";
    display: block;
    height: 3px;
    left: 50%;
    position: absolute;
    right: 50%;
    -webkit-transition: left 334ms cubic-bezier(0.4, 0, 1, 1), right 334ms cubic-bezier(0.4, 0, 1, 1);
    transition: left 334ms cubic-bezier(0.4, 0, 1, 1), right 334ms cubic-bezier(0.4, 0, 1, 1);
}

.navbar-nav li:hover:after {
    left: 5px;
    right: 5px;
    -webkit-transition: left 334ms cubic-bezier(0, 0, 0.2, 1), right 334ms cubic-bezier(0, 0, 0.2, 1);
    transition: left 334ms cubic-bezier(0, 0, 0.2, 1), right 334ms cubic-bezier(0, 0, 0.2, 1);
}

.navbar-inverse .navbar-brand {
    letter-spacing: 2px;
}

#logo {
    margin-top: -6px;
    padding: 5px;
    width: 40px;
    height: 40px;
}

.navbar-brand {
    padding: 11px 9px;
    font-size: 16px;
    line-height: 28px;
    margin-right: 50px;
}

.navbar-inverse .navbar-nav>li>a,
.navbar-inverse .navbar-brand {
    color: #fff;
}

.version-container {
    margin-bottom: 10px;
}

.version-container>label {
    position: relative;
    top: 2px;
}

.version-container .version-select {
    font-size: 12px;
    margin-bottom: 0;
    padding-bottom: 0;
    padding-left: 0;
    min-width: 215px;
    height: 30px;
}

.version-container .version-select option[selected] {
    font-weight: bold;
}

.icon-question,
.start-tour {
    text-indent: -9999px;
    background: url(../images/sprite-green.png) -199px -50px no-repeat;
    display: inline-block;
    width: 14px;
    height: 14px;
    cursor: pointer;
    top: 7px;
    position: relative;
}

.icon-question:hover,
.start-tour:hover {
    background: url(../images/sprite-green.png) -199px 0 no-repeat;
}

.floating-module {
    position: absolute;
    width: 100%;
    height: 100%;
    top: 0;
    left: 0;
    margin: 0;
}

.floating-module .module {
    position: relative;
    width: 250px;
    min-height: 300px;
    top: 45%;
    margin: auto;
    margin-top: -200px;
    -webkit-box-shadow: 0 2px 6px rgba(0, 0, 0, 0.3);
    -moz-box-shadow: 0 2px 6px rgba(0, 0, 0, 0.3);
    box-shadow: 0 2px 6px rgba(0, 0, 0, 0.3);
    -webkit-border-radius: 3px;
    -moz-border-radius: 3px;
    -ms-border-radius: 3px;
    -o-border-radius: 3px;
    border-radius: 3px;
}

.floating-module .logo-wrapper {
    text-align: center;
}

.floating-module .logo-wrapper p {
    font-size: 36px;
    text-transform: uppercase;
}

.login .form-control {
    margin-bottom: 10px;
    text-align: center;
}

.footer[data-page="error"],
.footer[data-page="login"] {
    position: fixed;
    bottom: 20px;
    width: 300px;
    left: 50%;
    margin-left: -150px;
}


/* Error page */

.error-page {
    text-align: center;
}

.error-page .logo-wrapper p {
    margin-bottom: 5px;
}

.error-page .error-code {
    margin: 0 0 5px 0;
    font-size: 72px;
    color: #f26c5f;
}

.error-page .error-status {
    font-size: 18px;
    margin-bottom: 15px;
}

.error-page .error-text {
    font-size: 12px;
}

.modal-list {
    list-style-type: none;
    padding-left: 0;
}

.modal-list li {
    padding: 10px 0;
}


/*
 * Plans page
 */

#create-plan {
    float: left;
    margin-top: -5px;
}

table.dataTable.no-footer {
    border: none;
    border-bottom: 1px solid #dddedf;
}

table.dataTable td.center,
table.dataTable th.center {
    text-align: center;
}

table.dataTable thead td {
    padding: 10px 0;
    border-bottom: 1px solid #dddedf;
    font-size: 13px;
}

table.dataTable tbody td {
    padding: 10px 0;
}

table.dataTable.display tbody td {
    border: none;
}

table.dataTable.display tr td:first-child {
    padding-left: 15px;
}

#filter-form .twitter-typeahead {
    width: 100%;
}

.messages #filter-form span[class*="col-"],
.incidents #filter-form span[class*="col-"] {
    padding-left: 5px;
    padding-right: 5px;
}

#filter-form .form-inline label {
    width: 38%;
}

#filter-form select.border-bottom {
    background-color: #fff;
    background-position: calc(100% - 5px) 13px;
}

#incidents-table_filter {
    padding-right:  30px;
    display: inline;
}
.dataTable tbody tr {
    cursor: pointer;
}


/* override datatables icons with bootstrap icons */

table.dataTable thead .sorting,
table.dataTable thead .sorting_desc,
table.dataTable thead .sorting_asc {
    background-image: none;
}

.dataTable .glyphicon.glyphicon-sort {
    font-size: .9em;
}

.dataTable .sorting_asc .glyphicon.glyphicon-sort:before {
    content: "\e155";
}

.dataTable .sorting_desc .glyphicon.glyphicon-sort:before {
    content: "\e156";
}

.dataTables_length select.form-control.border-bottom {
    width: auto;
    background-position: calc(100% - 5px) 13px;
}

#plans-table tbody tr td {
    vertical-align: top;
}

#plans-table tbody td sub {
    font-size: 80%;
}

.row.checkbox {
    margin-left: 0;
    margin-right: 0;
}

#filter-form .checkbox label {
    padding-left: 5px;
    margin-right: 10px;
    position: relative;
    top: 2px;
}


/*
 * Plan page
 */

#plan-name,
#plan-desc {
    width: 50%;
    margin-bottom: 10px;
    display: inline-block;
}

.badge[data-active="1"] {
    background: #9fc448;
    color: #fff;
    font-style: normal;
    font-weight: normal;
}

.badge[data-active="1"]:after {
    content: "Active";
    display: block;
    padding: 0 5px;
}

.badge[data-active="0"] {
    background: #d0d3d6;
    color: #fff;
    font-style: normal;
    font-weight: normal;
    cursor: pointer;
    min-width: 90px;
}

.badge[data-active="0"]:after {
    content: "Deactivated";
    display: block;
    padding: 0 5px;
}

.view-template .badge[data-active="0"]:hover,
.plan-details .badge[data-active="0"]:hover {
    background: #9fc448;
}

.view-template .badge[data-active="0"]:hover:after,
.plan-details .badge[data-active="0"]:hover:after {
    content: "Activate";
    display: block;
    padding: 0 5px;
}

.badge[data-active="draft"] {
    background: #f9e4a2;
    color: #fff;
    font-style: normal;
    font-weight: normal;
}

.badge[data-active="draft"]:after {
    content: "Draft";
    display: block;
    padding: 0 5px;
}

.badge.disabled {
    pointer-events: none;
}

.badge.disabled:after {
    content: "Activating.." !important;
}

.module.plan-steps {
    padding: 0;
    list-style-type: none;
}

#add-step,
#add-tracking-template,
#add-tracking-notification,
.step,
.plan-step {
    padding: 10px 20px;
    position: relative;
    list-style-type: none;
}

.plan-step {
    counter-increment: step-counter;
    padding-left: 120px;
}

.plan-step:before {
    content: counter(step-counter);
    font-size: 54px;
    top: 75px;
    left: 20px;
    position: absolute;
    color: #ccc;
    vertical-align: top;
    float: left;
    min-height: 200px;
    padding-left: 25px;
}

.plan-step[data-mode="edit"] {
    cursor: move;
}

.remove-step {
    font-size: 50px;
    top: 50%;
    right: 20px;
    margin-top: -16px;
    position: absolute;
    cursor: pointer;
    opacity: 0;
}

.step:hover .remove-step,
.plan-step:hover .remove-step {
    opacity: 1;
    -webkit-transition: opacity .5s;
    -moz-transition: opacity .5s;
}

.plan-step>span {
    display: inline-block;
}

.plan-step-number {
    font-size: 54px;
    margin: 0 10px;
    padding-top: 70px;
    color: #ccc;
    vertical-align: top;
}

#add-tracking-template,
#add-tracking-notification,
#add-step {
    text-align: center;
    color: #00b0e3;
    text-transform: uppercase;
    letter-spacing: 1px;
    cursor: pointer;
}

.add-btn h4 {
    margin: 10px 0;
}

.add-btn .icon-plus {
    display: inline-block;
    color: #00b0e3;
    border-color: #00b0e3;
}

.toggle-input {
    font-size: 14px;
    height: 32px;
    -webkit-user-select: none;
    user-select: none;
}

.toggle-input input[type="checkbox"] {
    opacity: 0;
    position: absolute;
}

.toggle-input input[type="checkbox"]+label {
    position: relative;
    padding: 8px 50px 0 0;
    line-height: 10px;
    color: #b0b2b5;
}

.toggle-input input[type="checkbox"]+label:before {
    content: "";
    display: block;
    position: absolute;
    width: 40px;
    height: 24px;
    top: 0;
    right: 0;
    left: auto;
    border-radius: 16px;
    transition: background-color .2s ease-out;
    background-color: #d0d3d6;
    border: 1px solid #a0a3a6;
    box-shadow: none;
}

.toggle-input input[type="checkbox"]:checked+label:before {
    background-color: #0084bf;
    border-color: transparent;
    box-shadow: 0 0 0 10px #0084BF inset;
}

.toggle-input input[type="checkbox"]+label:after {
    display: block;
    content: "";
    background-color: white;
    height: 20px;
    width: 20px;
    margin: 2px;
    position: absolute;
    top: 0px;
    right: 16px;
    left: auto;
    background-image: none;
    transform: scale(1);
    box-shadow: rgba(0, 0, 0, 0.0980392) 0px 0px 0px 1px, rgba(0, 0, 0, 0.2) 0px 2px 3px;
    border-radius: 13px;
    transition: all 0.3s ease;
}

.toggle-input input[type="checkbox"]:checked+label:after {
    transform: translateX(16px);
}

.test-plan-dynamic-targets {
    display: -webkit-flex;
    display: flex;
    flex-wrap: wrap;
}

.test-plan-dynamic-target {
    width: 45%;
    margin: 15px 0 15px 15px;
    padding: 14px;
    background-color: #fff;
    border: 1px solid #dddedf;
    border-radius: 5px;
}

.test-plan-dynamic-target .twitter-typeahead input.form-control {
    width: 100%;
}

.test-plan-dynamic-target .item-label {
    text-align: right;
    font-weight: normal;
    width: 20%;
    margin: 0;
    user-select: none;
    -webkit-user-select: none;
    -moz-user-select: none;
}

.plan-step-time {
    margin: 10px 15px 20px 30px;
    float: left;
    position: absolute;
    top: 130px;
    left: 0px;
    color: #ccc;
}

.plan-notification-add,
.plan-notification {
    width: 260px;
    min-height: 205px;
    margin: 15px 0 15px 15px;
    padding: 20px;
    background-color: #fff;
    float: left;
    border: 1px solid #dddedf;
    border-radius: 5px;
    position: relative;
}

.plan-notification p,
.test-plan-dynamic-target p {
    min-height: 35px;
    line-height: 35px;
}

.plan-notification select {
    text-transform: capitalize;
}

.plan-notification[data-dynamic="0"] .notification-dynamic {
    display: none;
}

.plan-notification[data-dynamic="1"] .notification-static {
    display: none;
}

.plan-notification .twitter-typeahead {
    font-size: .9em;
    width: 65%;
}

.plan-notification-add {
    padding-top: 64px;
    text-align: center;
    cursor: pointer;
    text-transform: uppercase;
    letter-spacing: 1px;
    color: #b0b2b5;
    line-height: 32px;
}

.plan-notification-remove {
    position: absolute;
    color: #b0b2b5;
    padding: 3px 5px;
    top: 0;
    right: 0;
    cursor: pointer;
    opacity: 0;
    -webkit-transition: opacity .5s;
}

.plan-notification:hover .plan-notification-remove {
    opacity: 1;
}

.plan-notification p {
    margin-bottom: 0;
}

.plan-notification select,
.plan-notification input {
    width: 65%;
    height: 31px;
    display: inline-block;
}

.plan-notification select:after {
    content: "\03bd";
    float: right;
    display: block;
}

.plan-notification .item-label {
    text-align: right;
    font-weight: normal;
    width: 33%;
    margin: 0;
    user-select: none;
    -webkit-user-select: none;
    -moz-user-select: none;
}

.plan-notification .item-content {
    width: 100%;
    text-overflow: ellipsis;
    overflow: hidden;
    white-space: nowrap;
}

#aggregation {
    line-height: 1;
    margin-top: 10px;
}

#tracking-notification .tracking-inner h4,
#aggregation h4,
#reprioritization h4 {
    color: #666;
    cursor: pointer;
    user-select: none;
    -webkit-user-select: none;
    -moz-user-select: none;
    margin-bottom: 0;
}

.aggregation-content {
    border-top: 1px solid #dddedf;
    display: none;
}

#tracking-notification .glyphicon-triangle-bottom,
#aggregation .glyphicon-triangle-bottom,
#reprioritization .glyphicon-triangle-bottom,
.application-dangerous-actions .glyphicon-triangle-bottom {
    font-size: 12px;
    left: -2px;
    top: 1px;
    transition: transform .2s;
    -webkit-transition: transform .2s;
    -moz-transition: transform .2s;
}

#tracking-notification .glyphicon-triangle-bottom,
#aggregation .glyphicon-triangle-bottom,
#reprioritization .glyphicon-triangle-bottom,
.application-dangerous-actions .glyphicon-triangle-bottom {
    transform: rotate(-90deg);
    -webkit-transform: rotate(-90deg);
    -moz-transform: rotate(-90deg);
}

#tracking-notification.active .glyphicon-triangle-bottom,
#aggregation.active .glyphicon-triangle-bottom,
#reprioritization.active .glyphicon-triangle-bottom,
.application-dangerous-actions.active .glyphicon-triangle-bottom {
    transform: rotate(0deg);
    -webkit-transform: rotate(0deg);
    -moz-transform: rotate(0deg);
}

#aggregation.active .aggregation-content {
    padding-top: 5px;
    margin-top: 5px;
    display: block;
}

#aggregation input {
    font-weight: bold;
    width: 40px;
    padding: 5px;
    text-align: center;
}

#reprio-controls input {
    font-weight: bold;
    width: 40px;
    padding: 5px;
    text-align: center;
}

#reprio-controls select {
    font-weight: bold;
    width: 100px;
    padding: 5px;
    text-align: left;
}

#reprio-controls button {
    margin-left: 10px;
}

#reprio-controls {
    padding: 25px;
}

#reprioritization-table tr:nth-child(even) {
    background: #f0f3f6;
}

#reprioritization.active #reprio-inner {
    display: block;
}

#reprioritization #reprio-inner {
    display: none;
}

#reprioritization {
    clear: both;
    padding: 0;
}

#reprioritization h4 {
    padding: 25px;
}

#tracking-notification {
    padding: 0;
}

#tracking-notification .tracking-target {
    padding: 5px 25px;
}

#tracking-notification .tracking-target .item-content {
    margin-right: 15px;
}

#tracking-notification .tracking-target label {
    font-size: 16px;
}

#tracking-notification .tracking-inner {
    padding: 25px;
}

#tracking-notification[data-view="true"] .tracking-inner h4 {
    cursor: auto;
}

#tracking-notification .tracking-settings {
    display: none;
}

#tracking-notification.active .tracking-settings {
    display: block;
}

#tracking-notification .template-steps {
    padding: 0;
    margin: 0;
}

#tracking-notification .tracking-settings select,
#tracking-notification span.tracking-settings,
#tracking-notification .tracking-settings b,
#tracking-notification .tracking-settings input {
    margin-right: 10px;
    width: auto;
    display: inline-block;
}

#tracking-notification .template-notification {
    min-width: 516px;
    min-height: 228px;
}

#tracking-notification .template-notification .notification-body {
    height: 200px;
}

#tracking-notification .template-notification[data-mode="email_subject"] {
    width: 1050px;
    height: 32px;
    min-height: 30px;
    margin-bottom: 0;
    background: none;
}

#tracking-notification .template-notification[data-mode="email_subject"] .notification-header,
#tracking-notification .template-notification[data-mode="email_subject"] .notification-body {
    height: 30px;
    float: left;
}

#tracking-notification .template-notification[data-mode="email_subject"] .notification-header {
    width: 15%;
    border-top-left-radius: 0;
    border-top-right-radius: 0;
    border-bottom: 1px solid #dddedf;
    border-right: none;
}

#tracking-notification .template-notification[data-mode="email_subject"] .notification-body {
    width: 85%;
    line-height: 25px;
    margin: 0;
}

#tracking-notification .template-notification[data-mode="email_subject"][data-view="true"] .notification-header {
    border-bottom: none;
    border-right: 1px solid #dddedf;
    line-height: 26px;
}

#tracking-notification .template-notification[data-mode="email_subject"][data-view="true"] .notification-body {
    border-top: none;
    line-height: 30px;
}


/* Drag styles */

.dragging {
    opacity: 0.5;
}

.drag-over {
    border: 1px dashed #ccc;
}


/*
 * Templates page
 */

.view-template input,
.view-template select,
.view-template textarea {
    margin-bottom: 10px;
}


/*
 * Template page
 */

.view-template h3.form-inline {
    margin-bottom: 0;
}

.module.template-steps {
    padding: 0;
}

.template-steps .step {
    list-style-type: none;
    padding: 0;
}

.view-template .template-steps .step .remove-step {
    right: 3px;
}

.step:first-child .template-header {
    border-top: none;
}

.step .template-header {
    background: #f0f3f6;
    padding: 5px 10px 5px 25px;
    margin-bottom: 5px;
    border-bottom: 1px solid #dddedf;
    border-top: 1px solid #dddedf;
}

.step .template-header label {
    font-size: 16px;
    margin-top: 5px;
    color: #333;
}

.step .template-header .form-control {
    width: auto;
    padding-right: 32px;
    margin-bottom: 0;
}

.step .variables {
    margin-left: 10px;
}

.step .variables .variable {
    font-weight: bold;
}

.step .variables .variable:after {
    content: ",";
    display: inline;
}

.step .variables .variable-required:after {
    content: "* ,";
    display: inline;
}

.step .variables .variable-required:last-of-type:after {
    content: "*";
    display: inline;
}

.step .variables .variable:not(.variable-required):last-of-type:after
/* Don't add trailing comma to last variable, and leave trailing asterisk if it's required. */

{
    content: "";
}

.step .template-body {
    padding: 0 10px;
}

.template-notification {
    width: 260px;
    min-height: 263px;
    margin: 10px 0 15px 18px;
    background-color: #fff;
    float: left;
    position: relative;
}

.template-notification[data-view="true"] {
    height: 250px;
    border: 1px solid #dddedf;
    border-radius: 2px;
}

.template-notification[data-view="true"] .notification-header {
    border: none;
}

.template-notification[data-view="true"] .notification-body {
    border-top: 1px solid #dddedf;
    height: 235px;
}

.template-notification[data-view="true"][data-mode="email"] .notification-body {
    height: 215px;
    /* less 20px for subject line */
}

.template-notification .notification-email-subject {
    overflow: hidden;
    height: 20px;
    white-space: nowrap;
    text-overflow: ellipsis;
    padding-right: 3px;
}

.template-notification .notification-header {
    background: #f0f3f6;
    color: #333;
    padding: 3px 0;
    text-align: center;
    text-transform: uppercase;
    border: 1px solid #dddedf;
    border-bottom: none;
    border-top-left-radius: 3px;
    border-top-right-radius: 3px;
}

.template-notification p,
.template-notification textarea,
.template-notification pre,
.template-notification input {
    margin-bottom: 0;
    border: 0;
    border-radius: 0;
    box-shadow: none;
    padding-left: 10px;
}

.template-notification textarea,
.template-notification input {
    border: 1px solid #dddedf;
}

.template-notification textarea:focus,
.template-notification input:focus,
.application-setting textarea:focus {
    outline: none;
    box-shadow: none;
}

.view-application .glyphicon-remove {
    display: none;
}

.view-application form {
    visibility: hidden;
}

.application-module .glyphicon-remove {
    cursor: pointer;
}

.application-module .label {
    margin-right: 3px;
}

.application-quota input {
    width: 60px;
    font-weight: bold;
    text-align: center;
    padding: 5px;
}

.application-quota input.quota-text {
    width: auto;
}

.application-quota span.twitter-typeahead {
    width: auto;
}


/* Make the typeahead menu appear a few pixels underneath the textbox, to avoid it overlapping slightly */

.application-quota .tt-menu {
    margin-top: 5px;
}


/* Hide garbled typeahead text */

.application-quota input.tt-hint {
    display: none;
}

.application-module input {
    width: 120px;
}

.application-module form {
    display: inline-block;
}

.application-dangerous-actions h4 {
    cursor: pointer;
    margin-bottom: 0;
}

.application-dangerous-actions.active .inner {
    display: block;
}

.application-dangerous-actions .inner {
    display: none;
}

#supported-modes-checkboxes input {
    width: auto;
}

#supported-modes-checkboxes {
    height: 30px;
}

#api-key-box {
    height: 22px;
}

#show-app-modal-button {
    margin-top: -5px;
}

.template-notification textarea {
    height: 250px;
}

.template-notification[data-mode="email"] textarea {
    height: 216px;
}

.template-notification[data-mode="sms"] textarea {
    height: 230px;
}


/* Add descending z-index for text area expansion */

.template-notification:nth-child(1) {
    z-index: 4;
}

.template-notification:nth-child(2) {
    z-index: 3;
}

.template-notification:nth-child(3) {
    z-index: 2;
}

.template-notification:nth-child(4) {
    z-index: 1;
}

#variables li {
    margin-left: 10px;
    list-style-type: none;
    display: inline;
}

.template-notification .preview-template {
    text-align: center;
    font-size: .9em;
    color: #00b0e3;
    cursor: pointer;
    padding: 5px 0;
    border: 1px solid #dddedf;
    border-top: none;
    border-bottom-left-radius: 3px;
    border-bottom-right-radius: 3px;
}

.template-notification .editor-tip {
    border: 1px solid #dddedf;
    border-top: none;
    text-align: center;
    font-size: .9em;
    height: 20px;
}

.template-notification .preview-template:hover {
    background-color: #f0f3f6;
}

.preview-subject {
    border-bottom: 1px solid #dddedf;
    margin-bottom: 5px;
    padding-bottom: 5px;
}

#preview-template-modal .modal-content {
    width: 700px;
}

#preview-template-modal .modal-body {
    padding-top: 5px;
}

#preview-template-modal .alert .close {
    display: none;
}


/*
 * Incidents page
 */

.claim-container {
    display: inline-block;
}

.incidents td.summary .icon-question {
    top: 3px;
}

.incidents td.summary .loader {
    display: none;
}

.incidents td.summary[data-loading="true"] .loader {
    margin: 0;
    display: inline-block;
    width: 13px;
    height: 13px;
    position: relative;
    border: 2px solid #008cc9;
    top: 3px;
    left: 3px;
}

.incidents .incident-summary {
    font-size: .8em;
    max-width: 100px;
    white-space: nowrap;
    text-overflow: ellipsis;
    overflow: hidden;
}

.incidents .incident-summary ul {
    list-style-type: none;
    padding: 0;
    margin: 0;
}

.incidents .incident-summary li {
    white-space: nowrap;
    text-overflow: ellipsis;
    overflow: hidden;
    overflow-y: auto;
    margin-right: 10px;
    max-height: 100px;
}

.incidents .incident-summary .tooltip-inner {
    white-space: nowrap;
    text-align: left;
    max-width: 100%;
    width: auto;
}

.incidents .incident-plan-name {
    max-width: 100px;
    white-space: nowrap;
    text-overflow: ellipsis;
    overflow: hidden;
}

.incident-info {
    font-size: 12px;
}

.incident-info a {
    margin: 0 15px;
}

.incident-details label {
    min-width: 100px;
}

.incident-details>div {
    border-bottom: 1px solid #f3f3f3;
    margin-bottom: 10px;
    max-height: 500px;
    overflow-y: auto;
}

.incident-details>div:last-child {
    border-bottom: none;
}

.incident-details #context-table {
    margin-left: 20px;
    margin-bottom: 20px;
}

.incident-details #context-table tr:nth-child(2n) {
    background: #f3f3f3;
}

.incident-details #context-table td {
    padding: 5px;
}

#comment-body {
    height: 100px;
}

.comment-display {
    padding-left: 10px;
}

.comment-header {
    margin-bottom: 3px;
    font-size: 12px;
    color: #777;
}

.comment-content {
    padding-left: 10px;
}

.comment-actions {
    margin-top: 10px;
}

#add-comment {
    display: none;
}

#re-escalate-container label {
    vertical-align: bottom;
}

.re-escalate-plan-container {
    max-height: 600px;
    overflow-y: scroll;
    overflow-x: hidden;
}

#re-escalate-container {
    margin-bottom: 10px;
}

#summary-info-modal .modal-content {
    width: 700px;
}

#summary-info-modal pre {
    border: 1px solid #ccc;
    background: #f9f9f9;
}

#summary-info-modal pre ul {
    list-style-type: none;
}


/*
 * Messages page
 */

#messages-table tbody {
    font-size: 13px;
}


/*
 * Message page
 */

.message-module {
    min-height: 250px;
}

.message-body {
    word-break: break-word;
    padding-bottom: 10px;
}


/*
 * User Settings page
 */

.user-settings .user-module {
    min-height: 850px;
}

.user-settings-name {
    text-transform: uppercase;
}

.user-settings-name h3 {
    line-height: 90px;
}

.user-settings-contact h4 {
    text-transform: uppercase;
    font-size: 16px;
    border-bottom: 1px solid #979797;
    padding-bottom: 5px;
    letter-spacing: 2px;
}

.user-settings-contact ul {
    list-style-type: none;
    margin: 0;
    padding: 0;
    font-size: 12px;
}

.user-settings-contact a {
    text-decoration: none;
}

.user-settings-contact ul a:hover {
    text-decoration: underline;
}

.user-settings-overrides {
    padding-top: 20px;
}

.user-settings table {
    margin-top: 12px;
    border: 1px solid #f3f3f3;
}

.user-settings table thead tr {
    background: #ddd;
}

.user-settings table thead td {
    padding: 3px 10px;
}

.user-settings table tbody td {
    padding: 5px 10px;
}

.user-settings table tr:nth-child(even) {
    background: #f0f3f6;
}

.priority.form-control {
    height: 28px;
    width: 120px;
}

.priority-name::first-letter,
.application-name::first-letter {
    text-transform: uppercase;
}

#priority-inner {
    width: 100%;
    margin-bottom: 20px;
}

#priority-table {
    width: 100%;
}

#priority-table .delete-column {
    width: 86px;
}

#priority-controls {
    margin-top: 10px;
    margin-bottom: 10px;
    margin-right: 20px;
}

#add-application-select {
    width: auto;
}

#timezone-select {
    width: auto;
}

#sms-override-select {
    width: auto;
}

.custom-notification.form-control {
    height: 28px;
    width: 120px;
}

#custom-inner {
    width: 100%;
    margin-bottom: 20px;
}

#custom-override-table {
    width: auto;
}

#custom-override-table .delete-column {
    width: 86px;
}

#custom-override-controls {
    margin-top: 10px;
    margin-bottom: 10px;
    margin-right: 20px;
}

#add-application-override-select {
    width: auto;
}

#custom-override-modal-table th {
    padding-right: 10px;
    padding-left: 10px;
}

.custom-modal-priority {
    width: auto;
}


/*
 * Applications page
 */

.application-setting {
    margin: 10px 0 15px 18px;
    background-color: #fff;
}

.application-setting textarea,
.application-setting pre {
    margin-bottom: 0;
    border-radius: 0;
    box-shadow: none;
    border: 1px solid #dddedf;
    height: 150px;
    width: 100%;
}

#application-email-incidents-table {
    width: auto;
    margin: 10px 0px;
}

#add-email-incident-form input {
    width: auto;
}

.application-email-incidents span.twitter-typeahead {
    width: auto;
}

#add-email-incident-form {
    /* Need this to make the email incidents text boxes line up, as the twitter typeahead
     skews everything */
    line-height: 33px;
}

.application-custom-category span.twitter-typeahead {
    width: auto;
}

#add-custom-category-form {
    /* Need this to make the email incidents text boxes line up, as the twitter typeahead
     skews everything */
    line-height: 33px;
}

#category-default-mode {
    width: 100px;
}

#add-default-mode-form select {
    width: 100px;
}

input.long-field {
    width: 550px;
}

input.auto-field {
    width: auto;
}

#add-address-form input {
    width: auto;
}


/*
 * Stats page
 */

.stats-module table {
    border: 1px solid #f3f3f3;
}

.stats-module table tr:nth-child(even) {
    background: #f0f3f6;
}


/*
 * Footer
 */

.footer {
    margin: 0;
    padding: 0;
    text-align: center;
}

.footer li {
    list-style-type: none;
    display: inline-block;
    border-left: 1px solid #333;
    padding: 0 8px;
    line-height: 1;
}

.footer li a {
    color: #333;
}

.footer li:first-child {
    border-left: none;
}


/*
 * Global add-ons
 */

.loader {
    border-width: 3px;
    margin-left: -10px;
    top: 25px;
    -webkit-animation-duration: 1s;
    -moz-animation-duration: 1s;
    animation-duration: 1s;
    -webkit-animation-name: pulsate;
    -moz-animation-name: pulsate;
    animation-name: pulsate;
    -webkit-animation-iteration-count: infinite;
    -moz-animation-iteration-count: infinite;
    animation-iteration-count: infinite;
    -webkit-animation-timing-function: ease-out;
    -moz-animation-timing-function: ease-out;
    animation-timing-function: ease-out;
    border-radius: 30px;
    filter: alpha(opacity=0);
    opacity: 1;
    border: 3px solid #008cc9;
    display: block;
    height: 30px;
    margin: 0 auto;
    width: 30px;
}

@keyframes pulsate {
    from {
        transform: scale(0.1, 0.1);
        opacity: .0
    }
    50% {
        opacity: 1.0
    }
    100% {
        transform: scale(1.2, 1.2);
        opacity: .0
    }
}

@-webkit-keyframes pulsate {
    from {
        -webkit-transform: scale(0.1, 0.1);
        opacity: .0
    }
    50% {
        opacity: 1.0
    }
    100% {
        -webkit-transform: scale(1.2, 1.2);
        opacity: .0
    }
}

.icon-plus {
    border: 1px solid #b0b2b5;
    border-radius: 50%;
    font-size: 32px;
    color: #b0b2b5;
    line-height: 43px;
    width: 50px;
    height: 50px;
    margin-bottom: 10px;
    display: block;
    margin: auto;
    font-style: normal;
}

.module {
    border: 1px solid #dddedf;
    background: #FFF;
    padding: 25px;
    margin-bottom: 10px;
}

.striped:nth-child(even) {
    background: #f0f3f6;
}

.module h4 {
    margin-top: 0;
}

.light {
    color: #b0b2b5;
}

.italic {
    font-style: italic;
}

.uppercase {
    text-transform: uppercase;
}

.btn.blue {
    color: #00b0e3;
    border-color: #00b0e3;
    margin: 0 3px;
}

h3 .btn-primary {
    margin: 0 3px;
}

.border-top {
    border-top: 1px solid #dddedf;
}

.border-bottom {
    border-bottom: 1px solid #dddedf;
}

.btn.blue:focus,
.btn.blue:active,
.btn.blue:hover {
    color: #0077B5;
    border-color: #0077B5;
    background: #fff;
}

.module-blue {
    border: none;
    border-top: 1px solid #979797;
    background: #f0f3f6;
}

.border-all {
    border: 1px solid #dddedf;
}

.chevron-down {
    display: inline-block;
    width: 18px;
    height: 18px;
    background: url('/static/images/chevron-down.png') no-repeat right 9px;
    background-size: 18px 8px;
}

.sub-header {
    padding-bottom: 10px;
    border-bottom: 1px solid #eee;
}

.invalid-input {
    border: 1px solid red !important;
}

.form-inline input.form-control {
    width: 38%;
}

.form-control.border-bottom {
    display: inline-block;
    -webkit-appearance: none;
    -moz-appearance: none;
    border-radius: 0;
    border: none;
    border-bottom: 1px solid #ccc;
    box-shadow: none;
}

input[type="number"].form-control.border-bottom {
    /* Fix for chrome bug where input shakes on focus */
    line-height: 1;
}

.form-control.border-bottom:focus {
    outline: none;
    box-shadow: none;
    border-bottom: 1px solid #008CC9;
}

select.border-bottom.form-control {
    background: url('/static/images/chevron-down.png') no-repeat right 13px;
    background-size: 14px 8px;
    padding-right: 20px;
}

select.form-control {
    padding: 4px 12px;
}

label {
    font-weight: normal;
}

pre {
    border: none;
    font-size: 11px;
    padding: 3px;
    background: inherit;
}

.required:after {
    content: "*";
    margin-left: 3px;
    color: #EC4339;
}

.alert {
    white-space: pre-wrap;
}

.alert.fixed {
    width: 400px;
    position: fixed;
    text-align: center;
    left: 50%;
    margin-left: -200px;
    z-index: 20;
}


/* Typeahead CSS */

span.twitter-typeahead {
    width: 70%;
}

.twitter-typeahead .tt-hint,
.twitter-typeahead .tt-input {
    width: 100%;
}

.twitter-typeahead .tt-cursor {
    background: #00b0e3;
    color: #fff;
}

.twitter-typeahead .tt-suggestion {
    padding-left: 10px;
}

.twitter-typeahead .tt-suggestion:hover {
    cursor: pointer;
    background: #00b0e3;
    color: #fff;
}

.twitter-typeahead .tt-menu {
    background: #fff;
    border: 1px solid #ccc;
    border-radius: 5px;
    padding: 10px 0;
    width: 100%;
}

.input-group span.twitter-typeahead {
    display: block !important;
}

.input-group span.twitter-typeahead .tt-dropdown-menu {
    top: 32px !important;
}

.input-group.input-group-lg span.twitter-typeahead .tt-dropdown-menu {
    top: 44px !important;
}

.input-group.input-group-sm span.twitter-typeahead .tt-dropdown-menu {
    top: 28px !important;
}