# Copyright (c) LinkedIn Corporation. All rights reserved. Licensed under the BSD-2 Clause license.
# See LICENSE in the project root for license information.


class Authenticator:
    def __init__(self, config):
        pass

    def authenticate(self, username, password):
        return True
